use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, Debug)]
pub struct Betting {
  pub _id: String,
  #[serde(rename = "vendorId")]
  pub vendor_id: i64,
  #[serde(rename = "vendorName")]
  pub vendor_name: String,
  #[serde(rename = "gameId")]
  pub game_id: i64,
  #[serde(rename = "gameName")]
  pub game_name: String,
  #[serde(rename = "gameCategory")]
  pub game_category: String,
  #[serde(rename = "gameType")]
  pub game_type: String,
  pub currency: String,
  pub cash: f64,
  #[serde(rename = "beforeCash")]
  pub before_cash: f64,
  #[serde(rename = "afterCash")]
  pub after_cash: f64,
  pub key: String,
  #[serde(rename = "refId")]
  pub ref_id: String,
  #[serde(rename = "oRefId")]
  pub o_ref_id: String,
  #[serde(rename = "groupKey")]
  pub group_key: Option<String>,
  #[serde(rename = "isBonus")]
  pub is_bonus: bool,
  #[serde(rename = "isPromo")]
  pub is_promo: bool,
  #[serde(rename = "isJackpot")]
  pub is_jackpot: bool,
  #[serde(rename = "siteUsername")]
  pub site_username: String,
  #[serde(rename = "type")]
  pub betting_type: String,
  pub category: String,
  #[serde(rename = "createdAt")]
  pub created_at: String,
  #[serde(rename = "utcCreatedAt")]
  pub utc_created_at: String,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct ListBettingsRequest {
  pub vendor_key: Option<String>,
  pub sdate: Option<String>,
  pub edate: Option<String>,
  pub group_key: Option<String>,
  pub username: Option<String>,
  pub limit: i64,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct _ListBettingsResponse {
  pub code: i64,
  pub msg: Option<String>,
  pub transactions: Option<Vec<Betting>>,
  #[serde(rename = "lastObjectId")]
  pub last_object_id: Option<String>,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct ListBettingsResponse {
  pub transactions: Vec<Betting>,
  pub last_object_id: Option<String>,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct GetPragmaticBettingRequest {
  pub id: String,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct _GetPragmaticBettingResponse {
  pub code: i64,
  pub msg: Option<String>,
  pub url: Option<String>,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct GetPragmaticBettingResponse {
  pub url: String,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct EvolutionBettingDetail {
  pub id: String,
  pub detail: String,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct GetEvolutionBettingDetailRequest {
  pub ids: String,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct _GetEvolutionBettingDetailResponse {
  pub code: i64,
  pub msg: Option<String>,
  pub transactions: Option<Vec<EvolutionBettingDetail>>,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct GetEvolutionBettingDetailResponse {
  pub transactions: Vec<EvolutionBettingDetail>,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct Statistic {
  #[serde(rename = "betAmount")]
  pub bet_amount: f64,
  #[serde(rename = "drawAmount")]
  pub draw_amount: f64,
  #[serde(rename = "cancelAmount")]
  pub cancel_amount: f64,
  #[serde(rename = "winAmount")]
  pub win_amount: f64,
  #[serde(rename = "revenueAmount")]
  pub revenue_amount: f64,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct GetStatisticRequest {
  pub vendor_key: Option<String>,
  pub group_key: Option<String>,
  pub date: Option<String>,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct _GetStatisticResponse {
  pub code: i64,
  pub msg: Option<String>,
  pub data: Option<Statistic>,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct GetStatisticResponse {
  pub data: Option<Statistic>,
}
