use super::models;
use beteran_common_rust as bcr;
use diesel::{result::Error, sql_query, RunQueryDsl};
use std::fmt::Write;

static MEMBER_COUNT_QUERY: &str = "
SELECT
  COUNT(akbh.id)
FROM api_kgon_betting_history as akbh
  INNER JOIN members _m
    ON _m.username = akbh.site_username
";

static MEMBER_QUERY: &str = "
SELECT
  akbh.id                   as akbh_id            ,
  akbh.vendor_id            as akbh_vendor_id     ,
  akbh.vendor_name          as akbh_vendor_name   ,
  akbh.game_id              as akbh_game_id       ,
  akbh.game_name            as akbh_game_name     ,
  akbh.game_category        as akbh_game_category ,
  akbh.game_type            as akbh_game_type     ,
  akbh.currency             as akbh_currency      ,
  akbh.cash                 as akbh_cash          ,
  akbh.before_cash          as akbh_before_cash   ,
  akbh.after_cash           as akbh_after_cash    ,
  akbh.key                  as akbh_key           ,
  akbh.ref_id               as akbh_ref_id        ,
  akbh.o_ref_id             as akbh_o_ref_id      ,
  akbh.group_key            as akbh_group_key     ,
  akbh.is_bonus             as akbh_is_bonus      ,
  akbh.is_promo             as akbh_is_promo      ,
  akbh.is_jackpot           as akbh_is_jackpot    ,
  akbh.site_username        as akbh_site_username ,
  akbh.betting_type         as akbh_betting_type  ,
  akbh.category             as akbh_category      ,
  akbh.created_at           as akbh_created_at    ,
  akbh.utc_created_at       as akbh_utc_created_at,

  _m.id                     as _m_id,
  _m.site_id                as _m_site_id,
  _m.member_class_id        as _m_member_class_id,  
  _m.member_level_id        as _m_member_level_id,    
  _m.username               as _m_username,  
  _m.password               as _m_password,  
  _m.nickname               as _m_nickname,
  _m.mobile_phone_number    as _m_mobile_phone_number,
  _m.state                  as _m_state,
  _m.state_changed_at       as _m_state_changed_at,
  _m.parent_member_id       as _m_parent_member_id,
  _m.child_member_count     as _m_child_member_count,
  _m.last_signined_ip       as _m_last_signined_ip,
  _m.last_signined_at       as _m_last_signined_at,
  _m.created_at             as _m_created_at,
  _m.updated_at             as _m_updated_at,
  _m.deleted_at             as _m_deleted_at

FROM api_kgon_betting_history as akbh
  INNER JOIN members _m
    ON _m.username = akbh.site_username
";

pub struct Composition {}

impl std::fmt::Debug for Composition {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Composition of members").finish()
  }
}

impl Composition {
  ///
  pub fn new() -> Composition {
    Composition {}
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: String,
  ) -> Result<Option<models::BettingHistoryModel>, Error> {
    let query = format!(
      "
    {}
    WHERE
      akbh.id = $1
    ",
      MEMBER_QUERY
    );

    match sql_query(query)
      .bind::<diesel::sql_types::Text, _>(id)
      .get_result::<models::BettingHistoryModel>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_by_member_id(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
  ) -> Result<Option<models::BettingHistoryModel>, Error> {
    let query = format!(
      "
    {}
    WHERE
      m.id = $1
    ",
      MEMBER_QUERY
    );

    match sql_query(query)
      .bind::<diesel::sql_types::Uuid, _>(member_id)
      .get_result::<models::BettingHistoryModel>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  fn get_ordery_by(&self, find_all: &models::FindAll) -> Result<String, Error> {
    let mut query_ordery_by = String::new();

    if let Some(sorts) = &find_all.sorts {
      for s in sorts {
        if !query_ordery_by.is_empty() {
          write!(&mut query_ordery_by, " , ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }

        let (property, sort) = match s {
          bcr::pagination::Sort::ASC(property) => (property.clone(), "ASC".to_string()),
          bcr::pagination::Sort::DESC(property) => (property.clone(), "DESC".to_string()),
        };

        let property = match property.as_str() {
          "member_id" => "_m.id".to_string(),
          "username" => "_m.username".to_string(),
          "nickname" => "_m.nickname".to_string(),
          "vendor_id" => "akbh.vendor_id".to_string(),
          "vendor_name" => "akbh.vendor_name".to_string(),
          "game_id" => "akbh.game_id".to_string(),
          "game_name" => "akbh.game_name".to_string(),
          "game_category" => "akbh.game_category".to_string(),
          "game_type" => "akbh.game_type".to_string(),
          "currency" => "akbh.currency".to_string(),
          "cash" => "akbh.cash".to_string(),
          "before_cash" => "akbh.before_cash".to_string(),
          "after_cash" => "akbh.after_cash".to_string(),
          "key" => "akbh.key".to_string(),
          "ref_id" => "akbh.ref_id".to_string(),
          "o_ref_id" => "akbh.o_ref_id".to_string(),
          "group_key" => "akbh.group_key".to_string(),
          "is_bonus" => "akbh.is_bonus".to_string(),
          "is_promo" => "akbh.is_promo".to_string(),
          "is_jackpot" => "akbh.is_jackpot".to_string(),
          "site_username" => "akbh.site_username".to_string(),
          "betting_type" => "akbh.betting_type".to_string(),
          "category" => "category.id".to_string(),
          "created_at" => "akbh.created_at".to_string(),
          "utc_created_at" => "akbh.utc_created_at".to_string(),
          _ => "".to_string(),
        };

        if !property.is_empty() {
          write!(&mut query_ordery_by, "{} {}", property, sort)
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
      }
    }

    Ok(query_ordery_by)
  }

  fn get_find_all(&self, find_all: &models::FindAll) -> Result<String, Error> {
    let mut query_where = String::new();

    if let Some(s) = &find_all.search {
      if let Some(sp) = &s.member_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "_m.id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.vendor_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.vendor_id = {}", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.vendor_name {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.vendor_name = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.vendor_name_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.vendor_name like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.game_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_id = {}", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.game_name {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_name = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.game_name_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_name like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.game_category {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_category = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.game_category_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_category like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.game_type {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_type = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.game_type_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_type like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.currency {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.currency = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.currency_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.currency like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.key {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.key = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.key_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.key like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.ref_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.ref_id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.ref_id_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.ref_id like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.o_ref_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.o_ref_id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.o_ref_id_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.o_ref_id like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.group_key {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.group_key = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.group_key_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.group_key like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.is_bonus {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.is_bonus = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.is_promo {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.is_promo = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.is_jackpot {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.is_jackpot = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.site_username {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.site_username = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.site_username_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.site_username like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.betting_type {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.betting_type = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.betting_type_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.betting_type like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.category {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.category = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.category_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.category like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
    }

    Ok(query_where)
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    use diesel::sql_types::BigInt;

    #[derive(QueryableByName)]
    struct Count {
      #[sql_type = "BigInt"]
      count: i64,
    }

    let mut query = String::new();
    write!(&mut query, "{}", MEMBER_COUNT_QUERY)
      .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;

    let query_where = self.get_find_all(find_all)?;
    if !query_where.is_empty() {
      write!(&mut query, " WHERE {}", query_where)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }

    println!("query: {}", query);

    match sql_query(query).get_result::<Count>(conn) {
      Ok(m) => Ok(m.count),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(0),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::BettingHistoryModel>, Error> {
    let mut query = String::new();
    write!(&mut query, "{}", MEMBER_QUERY)
      .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;

    let query_where = self.get_find_all(find_all)?;
    if !query_where.is_empty() {
      write!(&mut query, " WHERE {}", query_where)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }
    let query_ordery_by = self.get_ordery_by(find_all)?;
    if !query_ordery_by.is_empty() {
      write!(&mut query, " ORDER BY {}", query_ordery_by)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }

    println!("query: {}", query);

    match sql_query(query).get_results::<models::BettingHistoryModel>(conn) {
      Ok(m) => Ok(m),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(vec![]),
        _ => Err(e),
      },
    }
  }
}
